@@ -52,10 +52,10 @@ struct XfpmBrightnessPrivate
     gboolean		xrandr_has_hw;
     gboolean		helper_has_hw;
     
-    gint		max_level;
-    gint		current_level;
-    gint		min_level;
-    gint		step;
+    gint32		max_level;
+    gint32		current_level;
+    gint32		min_level;
+    gint32		step;
 };
 
 G_DEFINE_TYPE (XfpmBrightness, xfpm_brightness, G_TYPE_OBJECT)
@@ -152,7 +152,7 @@ xfpm_brightness_setup_xrandr (XfpmBrightness *brightness)
     Window window;
     gint major, minor, screen_num;
     int event_base, error_base;
-    gint min, max;
+    gint32 min, max;
     gboolean ret = FALSE;
     gint i;
     
@@ -333,7 +333,12 @@ xfpm_brightness_helper_get_value (const gchar *argument)
     if ( exit_status != 0 )
 	goto out;
 
-    value = atoi (stdout_data);
+    if ( stdout_data[0] == 'N' )
+        value = 0;
+    else if ( stdout_data[0] == 'Y' )
+        value = 1;
+    else
+        value = atoi (stdout_data);
 
 out:
     g_free (command);
@@ -344,9 +349,9 @@ out:
 static gboolean
 xfpm_brightness_setup_helper (XfpmBrightness *brightness)
 {
-    int ret;
+    gint32 ret;
 
-    ret = xfpm_brightness_helper_get_value ("get-max-brightness");
+    ret = (gint32) xfpm_brightness_helper_get_value ("get-max-brightness");
     g_debug ("xfpm_brightness_setup_helper: get-max-brightness returned %i", ret);
     if ( ret < 0 ) {
 	brightness->priv->helper_has_hw = FALSE;
@@ -363,12 +368,12 @@ xfpm_brightness_setup_helper (XfpmBrightness *brightness)
 static gboolean
 xfpm_brightness_helper_get_level (XfpmBrightness *brg, gint32 *level)
 {
-    int ret;
+    gint32 ret;
 
     if ( ! brg->priv->helper_has_hw )
 	return FALSE;
 
-    ret = xfpm_brightness_helper_get_value ("get-brightness");
+    ret = (gint32) xfpm_brightness_helper_get_value ("get-brightness");
 
     g_debug ("xfpm_brightness_helper_get_level: get-brightness returned %i", ret);
 
@@ -382,7 +387,7 @@ xfpm_brightness_helper_get_level (XfpmBrightness *brg, gint32 *level)
 }
 
 static gboolean
-xfpm_brightness_helper_set_level (XfpmBrightness *brg, gint level)
+xfpm_brightness_helper_set_level (XfpmBrightness *brg, gint32 level)
 {
     gboolean ret;
     GError *error = NULL;
@@ -405,6 +410,46 @@ out:
     return ret;
 }
 
+static gboolean
+xfpm_brightness_helper_get_switch (XfpmBrightness *brg, gint *brightness_switch)
+{
+    gint ret;
+
+    ret = xfpm_brightness_helper_get_value ("get-brightness-switch");
+
+    if ( ret >= 0 )
+    {
+	*brightness_switch = ret;
+	return TRUE;
+    }
+
+    return FALSE;
+}
+
+static gboolean
+xfpm_brightness_helper_set_switch (XfpmBrightness *brg, gint brightness_switch)
+{
+    gboolean ret;
+    GError *error = NULL;
+    gint exit_status = 0;
+    gchar *command = NULL;
+
+    command = g_strdup_printf ("pkexec " SBINDIR "/xfpm-power-backlight-helper --set-brightness-switch %i", brightness_switch);
+    ret = g_spawn_command_line_sync (command, NULL, NULL, &exit_status, &error);
+    if ( !ret )
+    {
+	g_warning ("xfpm_brightness_helper_set_switch: failed to set value: %s", error->message);
+	g_error_free (error);
+	goto out;
+    }
+    g_debug ("executed %s; retval: %i", command, exit_status);
+    ret = (exit_status == 0);
+
+out:
+    g_free (command);
+    return ret;
+}
+
 static gboolean
 xfpm_brightness_helper_up (XfpmBrightness *brightness, gint32 *new_level)
 {
@@ -417,7 +462,7 @@ xfpm_brightness_helper_up (XfpmBrightness *brightness, gint32 *new_level)
     if ( !ret )
 	return FALSE;
 
-    if ( hw_level == brightness->priv->max_level )
+    if ( hw_level >= brightness->priv->max_level )
     {
 	*new_level = brightness->priv->max_level;
 	return TRUE;
@@ -457,7 +502,7 @@ xfpm_brightness_helper_down (XfpmBrightness *brightness, gint32 *new_level)
     if ( !ret )
 	return FALSE;
     
-    if ( hw_level == brightness->priv->min_level )
+    if ( hw_level <= brightness->priv->min_level )
     {
 	*new_level = brightness->priv->min_level;
 	return TRUE;
@@ -613,7 +658,7 @@ gboolean xfpm_brightness_has_hw (XfpmBrightness *brightness)
     return brightness->priv->xrandr_has_hw || brightness->priv->helper_has_hw;
 }
 
-gint xfpm_brightness_get_max_level (XfpmBrightness *brightness)
+gint32 xfpm_brightness_get_max_level (XfpmBrightness *brightness)
 {
     return brightness->priv->max_level;
 }
@@ -659,3 +704,27 @@ gboolean xfpm_brightness_dim_down (XfpmBrightness *brightness)
     
     return ret;
 }
+
+gboolean xfpm_brightness_get_switch (XfpmBrightness *brightness, gint *brightness_switch)
+{
+    gboolean ret = FALSE;
+
+#ifdef ENABLE_POLKIT
+    if ( brightness->priv->helper_has_hw )
+    ret = xfpm_brightness_helper_get_switch (brightness, brightness_switch);
+#endif
+
+    return ret;
+}
+
+gboolean xfpm_brightness_set_switch (XfpmBrightness *brightness, gint brightness_switch)
+{
+    gboolean ret = FALSE;
+
+#ifdef ENABLE_POLKIT
+    if ( brightness->priv->helper_has_hw )
+	ret = xfpm_brightness_helper_set_switch (brightness, brightness_switch);
+#endif
+
+    return ret;
+}
