@@ -296,18 +296,14 @@ get_device_icon_name (UpClient *upower, UpDevice *device)
     }
     else
     {
-
-	if ( state == UP_DEVICE_STATE_EMPTY || state == UP_DEVICE_STATE_FULLY_CHARGED ||
-             state == UP_DEVICE_STATE_DISCHARGING || state == UP_DEVICE_STATE_CHARGING )
-            icon_name = icon_prefix;
-	else if (is_display_device (upower, device))
+        if (is_display_device (upower, device))
 	{
 	    /* Desktop system with no batteries */
 	    icon_name = g_strdup_printf ("%s", XFPM_AC_ADAPTER_ICON);
 	}
 	else
 	{
-	    icon_name = g_strdup_printf ("%smissing", icon_prefix);
+	    icon_name = icon_prefix;
 	}
     }
 
