@@ -39,6 +39,8 @@
 #define POLKIT_AUTH_SUSPEND_LOGIND	"org.freedesktop.login1.suspend"
 #define POLKIT_AUTH_HIBERNATE_LOGIND	"org.freedesktop.login1.hibernate"
 
+#define POLKIT_AUTH_SUSPEND_XFPM	"org.xfce.power.xfce4-pm-helper"
+#define POLKIT_AUTH_HIBERNATE_XFPM	"org.xfce.power.xfce4-pm-helper"
 
 
 const gchar *xfpm_power_translate_device_type (guint type);
