@@ -54,8 +54,10 @@
 static 	GtkBuilder *xml 			= NULL;
 static  GtkWidget  *nt				= NULL;
 
+static  GtkWidget *on_battery_display_blank	= NULL;
 static  GtkWidget *on_battery_dpms_sleep 	= NULL;
 static  GtkWidget *on_battery_dpms_off  	= NULL;
+static  GtkWidget *on_ac_display_blank		= NULL;
 static  GtkWidget *on_ac_dpms_sleep 		= NULL;
 static  GtkWidget *on_ac_dpms_off 		= NULL;
 static  GtkWidget *sideview                 = NULL; /* Sidebar tree view - all devices are in the sideview */
@@ -126,9 +128,15 @@ void        dpms_toggled_cb                        (GtkWidget *w,
 void        sleep_on_battery_value_changed_cb      (GtkWidget *w, 
 						    XfconfChannel *channel);
 
+void        display_blank_on_battery_value_changed_cb (GtkWidget *w,
+						       XfconfChannel *channel);
+
 void        off_on_battery_value_changed_cb        (GtkWidget *w, 
 						    XfconfChannel *channel);
 
+void        display_blank_on_ac_value_changed_cb   (GtkWidget *w,
+						    XfconfChannel *channel);
+
 void        sleep_on_ac_value_changed_cb           (GtkWidget *w, 
 						    XfconfChannel *channel);
 
@@ -399,12 +407,41 @@ dpms_toggled_cb (GtkWidget *w, XfconfChannel *channel)
     }
 }
 
+void
+display_blank_on_battery_value_changed_cb (GtkWidget *w, XfconfChannel *channel)
+{
+    GtkWidget *brg;
+    gint blank_value = (gint) gtk_range_get_value (GTK_RANGE (on_battery_display_blank));
+    gint sleep_value = (gint) gtk_range_get_value (GTK_RANGE (on_battery_dpms_sleep));
+    gint brightness_value;
+
+    if ( sleep_value != 0 )
+    {
+	if ( blank_value >= sleep_value )
+	{
+	    gtk_range_set_value (GTK_RANGE(on_battery_dpms_sleep), blank_value + 1 );
+	}
+    }
+
+    if ( lcd_brightness )
+    {
+	brg = GTK_WIDGET (gtk_builder_get_object (xml, "brightness-inactivity-on-battery"));
+	brightness_value = (gint) gtk_range_get_value (GTK_RANGE (brg));
+
+	if ( blank_value * 60 <= brightness_value && brightness_value != BRIGHTNESS_DISABLED)
+	{
+	    gtk_range_set_value (GTK_RANGE (brg), BRIGHTNESS_DISABLED);
+	}
+    }
+}
+
 void
 sleep_on_battery_value_changed_cb (GtkWidget *w, XfconfChannel *channel)
 {
     GtkWidget *brg;
-    gint off_value    = (gint)gtk_range_get_value (GTK_RANGE (on_battery_dpms_off));
-    gint sleep_value  = (gint)gtk_range_get_value (GTK_RANGE (w));
+    gint off_value    = (gint) gtk_range_get_value (GTK_RANGE (on_battery_dpms_off));
+    gint sleep_value  = (gint) gtk_range_get_value (GTK_RANGE (w));
+    gint blank_value  = (gint) gtk_range_get_value (GTK_RANGE (on_battery_display_blank));
     gint brightness_value;
     
     if ( off_value != 0 )
@@ -414,7 +451,15 @@ sleep_on_battery_value_changed_cb (GtkWidget *w, XfconfChannel *channel)
 	    gtk_range_set_value (GTK_RANGE(on_battery_dpms_off), sleep_value + 1 );
 	}
     }
-    
+
+    if ( blank_value != 0 )
+    {
+	if ( blank_value >= sleep_value )
+	{
+	    gtk_range_set_value (GTK_RANGE(on_battery_display_blank), sleep_value - 1 );
+	}
+    }
+
     if ( lcd_brightness )
     {
 	brg = GTK_WIDGET (gtk_builder_get_object (xml, "brightness-inactivity-on-battery"));
@@ -452,14 +497,43 @@ off_on_battery_value_changed_cb (GtkWidget *w, XfconfChannel *channel)
     }
 }
 
+void
+display_blank_on_ac_value_changed_cb (GtkWidget *w, XfconfChannel *channel)
+{
+    GtkWidget *brg;
+    gint blank_value = (gint) gtk_range_get_value (GTK_RANGE (on_ac_display_blank));
+    gint sleep_value = (gint) gtk_range_get_value (GTK_RANGE (on_ac_dpms_sleep));
+    gint brightness_value;
+
+    if ( sleep_value != 0 )
+    {
+	if ( blank_value >= sleep_value )
+	{
+	    gtk_range_set_value (GTK_RANGE(on_ac_dpms_sleep), blank_value + 1 );
+	}
+    }
+
+    if ( lcd_brightness )
+    {
+	brg = GTK_WIDGET (gtk_builder_get_object (xml, "brightness-inactivity-on-ac"));
+	brightness_value = (gint) gtk_range_get_value (GTK_RANGE (brg));
+
+	if ( blank_value * 60 <= brightness_value && brightness_value != BRIGHTNESS_DISABLED)
+	{
+	    gtk_range_set_value (GTK_RANGE (brg), BRIGHTNESS_DISABLED);
+	}
+    }
+}
+
 void
 sleep_on_ac_value_changed_cb (GtkWidget *w, XfconfChannel *channel)
 {
     GtkWidget *brg;
 
     gint brightness_value;
-    gint off_value    = (gint)gtk_range_get_value (GTK_RANGE(on_ac_dpms_off));
-    gint sleep_value  = (gint)gtk_range_get_value (GTK_RANGE(w));
+    gint off_value    = (gint)gtk_range_get_value (GTK_RANGE (on_ac_dpms_off));
+    gint sleep_value  = (gint)gtk_range_get_value (GTK_RANGE (w));
+    gint blank_value  = (gint)gtk_range_get_value (GTK_RANGE (on_ac_display_blank));
     
     if ( off_value > 60 || sleep_value > 60 )
     	return;
@@ -472,6 +546,14 @@ sleep_on_ac_value_changed_cb (GtkWidget *w, XfconfChannel *channel)
 	}
     }
 
+    if ( blank_value != 0 )
+    {
+	if ( blank_value >= sleep_value )
+	{
+	    gtk_range_set_value (GTK_RANGE(on_ac_display_blank), sleep_value - 1 );
+	}
+    }
+
     if ( lcd_brightness )
     {
 	brg = GTK_WIDGET (gtk_builder_get_object (xml, "brightness-inactivity-on-ac"));
@@ -1915,7 +1997,11 @@ static void dialog_response_cb (GtkDialog *dialog, gint response, XfconfChannel
     switch(response)
     {
 	case GTK_RESPONSE_HELP:
-	    xfce_dialog_show_help (GTK_WINDOW (dialog), "xfce4-power-manager", "start", NULL);
+#if LIBXFCE4UI_CHECK_VERSION(4, 11, 1)
+	    xfce_dialog_show_help_with_version (NULL, "xfce4-power-manager", "start", NULL, XFPM_VERSION_SHORT);
+#else
+	    xfce_dialog_show_help (NULL, "xfce4-power-manager", "start", NULL);
+#endif
 	    break;
 	default:
 	    settings_quit (GTK_WIDGET (dialog), channel);
@@ -1944,7 +2030,6 @@ xfpm_settings_dialog_new (XfconfChannel *channel, gboolean auth_suspend,
     GtkWidget *viewport;
     GtkWidget *hbox;
     GtkWidget *frame;
-    GtkWidget *on_battery_blank, *on_ac_blank;
     GtkListStore *list_store;
     GtkTreeViewColumn *col;
     GtkCellRenderer *renderer;
@@ -1977,16 +2062,16 @@ xfpm_settings_dialog_new (XfconfChannel *channel, gboolean auth_suspend,
     on_ac_dpms_sleep = GTK_WIDGET (gtk_builder_get_object (xml, "dpms-sleep-on-ac"));
     on_ac_dpms_off = GTK_WIDGET (gtk_builder_get_object (xml, "dpms-off-on-ac"));
 
-    on_battery_blank = GTK_WIDGET (gtk_builder_get_object (xml, "display-blank-on-battery"));
+    on_battery_display_blank = GTK_WIDGET (gtk_builder_get_object (xml, "display-blank-on-battery"));
     val = xfconf_channel_get_uint (channel, PROPERTIES_PREFIX ON_BATTERY_BLANK, 1);
-    gtk_range_set_value (GTK_RANGE (on_battery_blank), val);
+    gtk_range_set_value (GTK_RANGE (on_battery_display_blank), val);
     xfconf_g_property_bind (channel, PROPERTIES_PREFIX ON_BATTERY_BLANK,
-                            G_TYPE_INT, gtk_range_get_adjustment (GTK_RANGE (on_battery_blank)),
+                            G_TYPE_INT, gtk_range_get_adjustment (GTK_RANGE (on_battery_display_blank)),
                             "value");
 
-    on_ac_blank = GTK_WIDGET (gtk_builder_get_object (xml, "display-blank-on-ac"));
+    on_ac_display_blank = GTK_WIDGET (gtk_builder_get_object (xml, "display-blank-on-ac"));
     xfconf_g_property_bind (channel, PROPERTIES_PREFIX ON_AC_BLANK,
-                            G_TYPE_INT, gtk_range_get_adjustment (GTK_RANGE (on_ac_blank)),
+                            G_TYPE_INT, gtk_range_get_adjustment (GTK_RANGE (on_ac_display_blank)),
                             "value");
 
     dialog = GTK_WIDGET (gtk_builder_get_object (xml, "xfpm-settings-dialog"));
