@@ -49,6 +49,16 @@
 
 static void xfpm_backlight_finalize     (GObject *object);
 
+static void xfpm_backlight_get_property (GObject *object,
+                                         guint prop_id,
+                                         GValue *value,
+                                         GParamSpec *pspec);
+
+static void xfpm_backlight_set_property (GObject *object,
+                                         guint prop_id,
+                                         const GValue *value,
+                                         GParamSpec *pspec);
+
 #define ALARM_DISABLED 9
 
 #define XFPM_BACKLIGHT_GET_PRIVATE(o) \
@@ -71,12 +81,25 @@ struct XfpmBacklightPrivate
     gint32          last_level;
     gint32 	    max_level;
     
+    gint            brightness_switch;
+    gint            brightness_switch_save;
+    gboolean        brightness_switch_initialized;
+
     gboolean        dimmed;
     gboolean	    block;
 };
 
+enum
+{
+    PROP_0,
+    PROP_BRIGHTNESS_SWITCH,
+    PROP_BRIGHTNESS_SWITCH_SAVE,
+    N_PROPERTIES
+};
+
 G_DEFINE_TYPE (XfpmBacklight, xfpm_backlight, G_TYPE_OBJECT)
 
+
 static void
 xfpm_backlight_dim_brightness (XfpmBacklight *backlight)
 {
@@ -200,19 +223,19 @@ xfpm_backlight_button_pressed_cb (XfpmButton *button, XfpmButtonKey type, XfpmBa
     gint32 level;
     gboolean ret = TRUE;
     
-    gboolean enable_brightness, show_popup;
+    gboolean handle_brightness_keys, show_popup;
     
     g_object_get (G_OBJECT (backlight->priv->conf),
-                  ENABLE_BRIGHTNESS_CONTROL, &enable_brightness,
-		  SHOW_BRIGHTNESS_POPUP, &show_popup,
+                  HANDLE_BRIGHTNESS_KEYS, &handle_brightness_keys,
+                  SHOW_BRIGHTNESS_POPUP, &show_popup,
                   NULL);
     
     if ( type != BUTTON_MON_BRIGHTNESS_UP && type != BUTTON_MON_BRIGHTNESS_DOWN )
 	return; /* sanity check, can this ever happen? */
 
     backlight->priv->block = TRUE;
-    if ( !enable_brightness )
-	ret = xfpm_brightness_get_level (backlight->priv->brightness, &level);
+    if ( !handle_brightness_keys )
+        ret = xfpm_brightness_get_level (backlight->priv->brightness, &level);
     else
     {
 	if ( type == BUTTON_MON_BRIGHTNESS_UP )
@@ -285,8 +308,28 @@ xfpm_backlight_class_init (XfpmBacklightClass *klass)
 {
     GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+    object_class->get_property = xfpm_backlight_get_property;
+    object_class->set_property = xfpm_backlight_set_property;
     object_class->finalize = xfpm_backlight_finalize;
 
+    g_object_class_install_property (object_class,
+                                     PROP_BRIGHTNESS_SWITCH,
+                                     g_param_spec_int (BRIGHTNESS_SWITCH,
+                                                       NULL, NULL,
+                                                       -1,
+                                                       1,
+                                                       -1,
+                                                       G_PARAM_READWRITE));
+
+    g_object_class_install_property (object_class,
+                                     PROP_BRIGHTNESS_SWITCH_SAVE,
+                                     g_param_spec_int (BRIGHTNESS_SWITCH_SAVE,
+                                                       NULL, NULL,
+                                                       -1,
+                                                       1,
+                                                       -1,
+                                                       G_PARAM_READWRITE));
+
     g_type_class_add_private (klass, sizeof (XfpmBacklightPrivate));
 }
 
@@ -305,6 +348,7 @@ xfpm_backlight_init (XfpmBacklight *backlight)
     backlight->priv->power    = NULL;
     backlight->priv->dimmed = FALSE;
     backlight->priv->block = FALSE;
+    backlight->priv->brightness_switch_initialized = FALSE;
     
     if ( !backlight->priv->has_hw )
     {
@@ -313,12 +357,67 @@ xfpm_backlight_init (XfpmBacklight *backlight)
     }
     else
     {
+	gboolean ret, handle_keys;
+
 	backlight->priv->idle   = egg_idletime_new ();
 	backlight->priv->conf   = xfpm_xfconf_new ();
 	backlight->priv->button = xfpm_button_new ();
 	backlight->priv->power    = xfpm_power_get ();
 	backlight->priv->notify = xfpm_notify_new ();
 	backlight->priv->max_level = xfpm_brightness_get_max_level (backlight->priv->brightness);
+	backlight->priv->brightness_switch = -1;
+
+	xfconf_g_property_bind (xfpm_xfconf_get_channel(backlight->priv->conf),
+							PROPERTIES_PREFIX BRIGHTNESS_SWITCH, G_TYPE_INT,
+							G_OBJECT(backlight), BRIGHTNESS_SWITCH);
+
+	ret = xfpm_brightness_get_switch (backlight->priv->brightness,
+									  &backlight->priv->brightness_switch);
+
+	if (ret)
+	g_object_set (G_OBJECT (backlight),
+				  BRIGHTNESS_SWITCH,
+				  backlight->priv->brightness_switch,
+				  NULL);
+	backlight->priv->brightness_switch_initialized = TRUE;
+
+	/*
+	 * If power manager has crashed last time, the brightness switch
+	 * saved value will be set to the original value. In that case, we
+	 * will use this saved value instead of the one found at the
+	 * current startup so the setting is restored properly.
+	 */
+	backlight->priv->brightness_switch_save =
+		xfconf_channel_get_int (xfpm_xfconf_get_channel(backlight->priv->conf),
+								PROPERTIES_PREFIX BRIGHTNESS_SWITCH_SAVE,
+								-1);
+
+	if (backlight->priv->brightness_switch_save == -1)
+	{
+	if (!xfconf_channel_set_int (xfpm_xfconf_get_channel(backlight->priv->conf),
+								 PROPERTIES_PREFIX BRIGHTNESS_SWITCH_SAVE,
+								 backlight->priv->brightness_switch))
+	g_critical ("Cannot set value for property %s\n", BRIGHTNESS_SWITCH_SAVE);
+
+	backlight->priv->brightness_switch_save = backlight->priv->brightness_switch;
+	}
+	else
+	{
+	g_warning ("It seems the kernel brightness switch handling value was "
+			   "not restored properly on exit last time, xfce4-power-manager "
+			   "will try to restore it this time.");
+	}
+
+    /* check whether to change the brightness switch */
+	handle_keys = xfconf_channel_get_bool (xfpm_xfconf_get_channel(backlight->priv->conf),
+										   PROPERTIES_PREFIX HANDLE_BRIGHTNESS_KEYS,
+										   TRUE);
+	backlight->priv->brightness_switch = handle_keys ? 0 : 1;
+	g_object_set (G_OBJECT (backlight),
+				  BRIGHTNESS_SWITCH,
+				  backlight->priv->brightness_switch,
+				  NULL);
+
 	g_signal_connect (backlight->priv->idle, "alarm-expired",
                           G_CALLBACK (xfpm_backlight_alarm_timeout_cb), backlight);
         
@@ -344,6 +443,62 @@ xfpm_backlight_init (XfpmBacklight *backlight)
     }
 }
 
+static void
+xfpm_backlight_get_property (GObject *object,
+                             guint prop_id,
+                             GValue *value,
+                             GParamSpec *pspec)
+{
+    XfpmBacklight *backlight = XFPM_BACKLIGHT (object);
+
+    switch (prop_id)
+    {
+    case PROP_BRIGHTNESS_SWITCH:
+        g_value_set_int (value, backlight->priv->brightness_switch);
+        break;
+    case PROP_BRIGHTNESS_SWITCH_SAVE:
+        g_value_set_int (value, backlight->priv->brightness_switch_save);
+        break;
+    default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+        break;
+    }
+}
+
+static void
+xfpm_backlight_set_property (GObject *object,
+                             guint prop_id,
+                             const GValue *value,
+                             GParamSpec *pspec)
+{
+    XfpmBacklight *backlight = XFPM_BACKLIGHT (object);
+    gboolean ret;
+
+    switch (prop_id)
+    {
+	case PROP_BRIGHTNESS_SWITCH:
+        backlight->priv->brightness_switch = g_value_get_int (value);
+        if (!backlight->priv->brightness_switch_initialized)
+            break;
+        ret = xfpm_brightness_set_switch (backlight->priv->brightness,
+                                          backlight->priv->brightness_switch);
+        if (!ret)
+            g_warning ("Unable to set the kernel brightness switch parameter to %d.",
+                       backlight->priv->brightness_switch);
+        else
+            g_message ("Set kernel brightness switch to %d",
+                       backlight->priv->brightness_switch);
+
+	    break;
+	case PROP_BRIGHTNESS_SWITCH_SAVE:
+        backlight->priv->brightness_switch_save = g_value_get_int (value);
+        break;
+    default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+        break;
+    }
+}
+
 static void
 xfpm_backlight_finalize (GObject *object)
 {
@@ -353,14 +508,37 @@ xfpm_backlight_finalize (GObject *object)
 
     xfpm_backlight_destroy_popup (backlight);
 
-    if ( backlight->priv->brightness )
-	g_object_unref (backlight->priv->brightness);
-
     if ( backlight->priv->idle )
 	g_object_unref (backlight->priv->idle);
 
     if ( backlight->priv->conf )
-	g_object_unref (backlight->priv->conf);
+    {
+    /* restore video module brightness switch setting */
+    if ( backlight->priv->brightness_switch_save != -1 )
+    {
+    gboolean ret =
+        xfpm_brightness_set_switch (backlight->priv->brightness,
+                                    backlight->priv->brightness_switch_save);
+    /* unset the xfconf saved value after the restore */
+    if (!xfconf_channel_set_int (xfpm_xfconf_get_channel(backlight->priv->conf),
+                                 PROPERTIES_PREFIX BRIGHTNESS_SWITCH_SAVE, -1))
+    g_critical ("Cannot set value for property %s\n", BRIGHTNESS_SWITCH_SAVE);
+
+    if (ret)
+    {
+    backlight->priv->brightness_switch = backlight->priv->brightness_switch_save;
+    g_message ("Restored brightness switch value to: %d", backlight->priv->brightness_switch);
+    }
+    else
+    g_warning ("Unable to restore the kernel brightness switch parameter to its original value, "
+               "still resetting the saved value.");
+    }
+
+    g_object_unref (backlight->priv->conf);
+    }
+
+    if ( backlight->priv->brightness )
+	g_object_unref (backlight->priv->brightness);
 
     if ( backlight->priv->button )
 	g_object_unref (backlight->priv->button);
